#!/usr/bin/perl
use strict;
use warnings;
use Bio::EnsEMBL::Registry;

my $registry = 'Bio::EnsEMBL::Registry';

$registry->load_registry_from_db(
  -host => 'ensembldb.ensembl.org',
  -user => 'anonymous'
);

my @db_adaptors = @{ $registry->get_all_DBAdaptors() };   

VISTA_Configurator("Human", "ENSG00000185960", 10000);

#VISTA_Configurator("Mouse", "ENSMUSG00000037025", 10000);
#VISTA_Configurator("Rat", "ENSRNOG00000013133", 10000);
#VISTA_Configurator("Guinea Pig", "ENSCPOG00000027068", 10000);
#VISTA_Configurator("Squirrel", "ENSSTOG00000020150", 10000);
VISTA_Configurator("Human", "ENSG00000125798", 10000);
#VISTA_Configurator("Gorilla", "ENSGGOG00000006368", 10000);

####
sub VISTA_Configurator {
  my($spName, $geneID, $extension) = @_;  
  
  my $spNameP = $spName;
  $spNameP =~ s/ //g;

  my $outSeqFile = "$spNameP\_$geneID.fa";
  my $outAnnFile = "$spNameP\_$geneID.txt";

  my $slice_adaptor = $registry->get_adaptor( $spName, 'Core', 'Slice' );
  my $slice         = $slice_adaptor->fetch_by_gene_stable_id($geneID, $extension);

  my $sliceMyGene  = $slice_adaptor->fetch_by_gene_stable_id($geneID);
  my @myGene = @{ $sliceMyGene->get_all_Genes() };
  my $myGene = shift@myGene;
  my $strand_myGene = $myGene->strand();

  ### Position in the genome
  print "$spNameP, $geneID, ".$myGene->coord_system_name(), ' ', $myGene->seq_region_name(), ' ';
  # Get and display the feature's position on the sequence region
  printf( "%d-%d (%+d)\n",
    $myGene->seq_region_start(),
    $myGene->seq_region_end(),
    $myGene->seq_region_strand() );

  #### Sequence
  my $seq          = $slice->seq();
  $seq = &revcomp($seq) if $strand_myGene < 0;
  my $orientation = "";
  if($strand_myGene > 0) {
    $orientation = "For";
  } else {
    $orientation = "Rev";
  }
  open(OUTS, ">$outSeqFile");
  print OUTS ">$spNameP$orientation\n$seq\n";
  close OUTS;
  
=pod ######### アノテーション情報を得ることは，この行頭に # を入れてコメントアウト

  #### Annotation
  my @genes = @{ $slice->get_all_Genes() };
  
  open(OUTA, ">$outAnnFile");
  while ( my $gene = shift @genes ) {
    my $name_gene        = $gene->external_name();                 
    my $start_gene       = $gene->start();                  
    my $end_gene         = $gene->end();
    my $strand_gene      = $gene->strand();                 
  
    my @transcripts = @{ $gene->get_all_Transcripts() };
  
    while ( my $transcript = shift @transcripts ) {
        print OUTA "> $start_gene $end_gene $name_gene\n";
  
        my $fiv_utr = $transcript->five_prime_utr_Feature();
        my $start_fiv_utr = $fiv_utr->start();
        my $end_fiv_utr   = $fiv_utr->end();
        if ($strand_myGene < 0){
          $start_fiv_utr    = length($seq)- $fiv_utr->end() + 1;
          $end_fiv_utr      = length($seq)- $fiv_utr->start() + 1;
        }
        print OUTA "$start_fiv_utr $end_fiv_utr UTR\n";
  
        my @exons = @{ $transcript->get_all_Exons() };
        while ( my $exon = shift @exons ) {
            my $start_exon = $exon ->start();
            my $end_exon   = $exon ->end();
            if ($strand_myGene < 0){
              $start_exon = length($seq)- $exon ->end() + 1 ;
              $end_exon   = length($seq)- $exon ->start() + 1;
            }
            print OUTA "$start_exon $end_exon exon\n";
        }
  
        my $thr_utr = $transcript->three_prime_utr_Feature();
        my $start_thr_utr = $thr_utr->start();
        my $end_thr_utr   = $thr_utr->end();
        if ($strand_myGene < 0){
          $start_thr_utr    = length($seq)- $thr_utr->end() + 1;
          $end_thr_utr      = length($seq)- $thr_utr->start() + 1;
        }
        print OUTA "$start_thr_utr $end_thr_utr UTR\n";
  
        print OUTA "\n";
    }
    print OUTA "\n";
  }  
  close OUTA;
=cut ######### アノテーション情報を得ることは，この行頭に # を入れてコメントアウト

}

sub revcomp {
    my $seqSR = shift;

    $seqSR = uc($seqSR);
    $seqSR =~ tr/ATGC/TACG/;
    $seqSR = reverse($seqSR);

    return $seqSR;
}